﻿Imports System.Globalization
Imports System.Reflection
Imports Syncfusion.UI.Xaml.Charts
Imports sfChartTest
Imports System.Windows.Threading
Imports System.Collections.ObjectModel

Class MainWindow

    Dim TestColl As ViewModel
    Dim BaseTicker As Ticker

    Dim XAxis As CustomDateTimeCategoryXAxis

    Dim YAxisPrice As CustomLogarithmicYAxis
    Dim YAxisVolume1 As CustomNumericalYAxis
    Dim YAxisVolume2 As CustomNumericalYAxis

    Dim SeriesPrice As New FastSteplineBitmapSeriesExt
    Dim SeriesVolume1 As New FastSteplineBitmapSeriesExt
    Dim SeriesVolume2 As New FastSteplineBitmapSeriesExt

    Dim scrollBar As SfChartResizableBar
    Dim zoomDays As Integer

    Dim BaseBrush As New SolidColorBrush(Color.FromArgb(255, 100, 200, 255))



    Sub New()

        ' This call is required by the designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.

        DataContext = Me

        TestColl = New ViewModel
        BaseTicker = TestColl.Tickers(0)

        Axes()
        Series()
        Annotations()


        'SET NO ANTIALIAS
        Dim image As Image = GetPrivateField(chart, "fastRenderDevice")
        If (Not (image) Is Nothing) Then
            RenderOptions.SetBitmapScalingMode(image, BitmapScalingMode.NearestNeighbor)
        End If

        ZoomX()

    End Sub

    Private Function GetPrivateField(ByVal chart As SfChart, ByVal name As String) As Image
        Dim info As FieldInfo = chart.GetType().GetField(name, BindingFlags.Instance Or (BindingFlags.NonPublic Or BindingFlags.Public))
        If (Not (info) Is Nothing) Then
            Return CType(info.GetValue(chart), Image)
        End If
        Return Nothing
    End Function

    Sub Axes()

        'PRIMARY AXIS
        XAxis = chart.PrimaryAxis
        XAxis.Name = "XAxis"

        'Y AXIS PRICE
        YAxisPrice = New CustomLogarithmicYAxis
        YAxisPrice.Name = "YAxisPrice"
        YAxisPrice.LogarithmicBase = LogBase
        YAxisPrice.FontSize = 12
        YAxisPrice.ShowGridLines = False
        YAxisPrice.Header = "Price"
        SfChart.SetRow(YAxisPrice, 2) 'row starts from 0 at the bottom

        'Y AXIS VOLUME 1
        YAxisVolume1 = New CustomNumericalYAxis
        YAxisVolume1.Name = "YAxisVolume1"
        YAxisVolume1.Minimum = 0
        YAxisVolume1.FontSize = 12
        YAxisVolume1.ShowGridLines = False
        YAxisVolume1.Visibility = Visibility.Visible
        YAxisVolume1.Header = "Volume 1"
        SfChart.SetRow(YAxisVolume1, 1) 'row starts from 0 at the bottom

        'Y AXIS VOLUME 2
        YAxisVolume2 = New CustomNumericalYAxis
        YAxisVolume2.Name = "YAxisVolume2"
        YAxisVolume2.Minimum = 0
        YAxisVolume2.FontSize = 12
        YAxisVolume2.ShowGridLines = False
        YAxisVolume2.Visibility = Visibility.Visible
        YAxisVolume2.Header = "Volume 2"
        SfChart.SetRow(YAxisVolume2, 0) 'row starts from 0 at the bottom

        'INTERVAL
        XAxis.IntervalType = DateTimeIntervalType.Days
        XAxis.Interval = 1
        XAxis.LabelFormat = "dd"
        chart.Header = "Days"

    End Sub

    Sub Series()

        'SERIES PRICE
        With SeriesPrice
            .Name = "SeriesPrice"
            .XBindingPath = "DateTime"
            .YBindingPath = "Price"
            .Interior = BaseBrush
            .StrokeThickness = 1
            .YAxis = YAxisPrice
            .ItemsSource = TestColl.Tickers(0).Minutes
        End With
        chart.Series.Add(SeriesPrice)

        'SERIES VOLUME 1
        With SeriesVolume1
            .Name = "SeriesVolume1"
            .XBindingPath = "DateTime"
            .YBindingPath = "Volume1"
            .StrokeThickness = 1
            .Interior = BaseBrush
            .YAxis = YAxisVolume1
            .ItemsSource = TestColl.Tickers(0).Minutes
        End With
        chart.Series.Add(SeriesVolume1)

        'SERIES VOLUME 2
        With SeriesVolume2
            .Name = "SeriesVolume2"
            .XBindingPath = "DateTime"
            .YBindingPath = "Volume2"
            .StrokeThickness = 1
            .Interior = BaseBrush
            .YAxis = YAxisVolume2
            .ItemsSource = TestColl.Tickers(0).Minutes
        End With
        chart.Series.Add(SeriesVolume2)

    End Sub

    Sub Annotations()

        For i = 0 To BaseTicker.Minutes.Count - 1

            If BaseTicker.Minutes(i).DateTime.TimeOfDay = TimeSpan.Parse("09:30:00") Then

                Dim newVerticalLineAnnotation As New VerticalLineAnnotation
                With newVerticalLineAnnotation
                    .CoordinateUnit = CoordinateUnit.Axis
                    .XAxisName = XAxis.Name
                    ' .YAxisName = YAxisPrice.Name
                    .X1 = i
                    .StrokeThickness = 1
                    .ShowLine = True
                    .YAxisName = "secondaryAxis"
                    .Stroke = New SolidColorBrush(Colors.Yellow)
                    .ShowAxisLabel = False
                    .EnableClipping = False
                End With
                chart.Annotations.Add(newVerticalLineAnnotation)
            End If
        Next

    End Sub

    Private Sub btZoomAll_Click(sender As Object, e As RoutedEventArgs) Handles btZoomAll.Click

        zoomDays = 0
        ZoomX()

    End Sub

    Private Sub btZoom3days_Click(sender As Object, e As RoutedEventArgs) Handles btZoom3days.Click

        zoomDays = 3
        ZoomX()

    End Sub

    Private Sub btZoom2days_Click(sender As Object, e As RoutedEventArgs) Handles btZoom2days.Click

        zoomDays = 2
        ZoomX()

    End Sub

    Private Sub btZoom1day_Click(sender As Object, e As RoutedEventArgs) Handles btZoom1day.Click

        zoomDays = 1
        ZoomX()

    End Sub

    Sub ZoomX()

        If zoomDays > 0 Then

            XAxis.ZoomPosition = 1 - (zoomDays / BaseTicker.TotalDays)
            XAxis.ZoomFactor = zoomDays / BaseTicker.TotalDays

            XAxis.EnableScrollBar = True
            scrollBar.RangeEnd = 1

        Else 'zoom all

            XAxis.ZoomPosition = 0
            XAxis.ZoomFactor = 1
            XAxis.EnableScrollBar = False

        End If

    End Sub

    Private Sub chart_Loaded(sender As Object, e As RoutedEventArgs) Handles chart.Loaded
        scrollBar = CType(chart.PrimaryAxis.GetType.GetField("sfChartResizableBar", BindingFlags.NonPublic Or BindingFlags.GetField Or BindingFlags.Instance).GetValue(chart.PrimaryAxis), SfChartResizableBar)
    End Sub


End Class

'Custom date time category axis created for overriding the labels generation method.
Class CustomDateTimeCategoryXAxis
    Inherits DateTimeCategoryAxis

    'Overriding this method can generate the labels As required In accordance With the data.
    Protected Overrides Sub GenerateVisibleLabels()

        If (IntervalType = DateTimeIntervalType.Days) Then

            Dim chartArea As ChartBase
            chartArea = CType((Me.GetType().GetProperty("Area", (BindingFlags.Instance + BindingFlags.NonPublic + BindingFlags.GetProperty)).GetValue(Me)), ChartBase)

            Dim actualSeries As ChartSeriesBase
            actualSeries = chartArea.VisibleSeries(0)

            If (Not (actualSeries) Is Nothing) Then

                Dim actualXValues = CType(actualSeries.GetType().GetProperty("ActualXValues", BindingFlags.GetProperty Or BindingFlags.NonPublic Or BindingFlags.Instance).GetValue(actualSeries), System.Collections.IEnumerable)

                Dim xValuesCount = CType(actualXValues, List(Of Double))

                If ((Not (xValuesCount) Is Nothing) AndAlso (xValuesCount.Count > 0)) Then

                    Dim interval As Double = Me.Interval
                    Dim position As Double = VisibleRange.Start - VisibleRange.Start Mod interval
                    Dim xValues = CType(actualXValues, List(Of Double))
                    Dim xStartValue = xValues(0).FromOADate()

                    Dim date1 As DateTime = xStartValue.Date

                    Do While (position <= VisibleRange.End)
                        Dim pos As Integer = CType(Math.Round(position), Integer)

                        If (VisibleRange.Inside(position) AndAlso pos > -1 AndAlso (pos < xValues.Count)) Then

                            Dim xValue = xValues(pos).FromOADate()
                            If (xValue.Date > date1) Then
                                date1 = xValue.Date
                            End If

                            If (xValue.Date = date1) Then
                                Dim obj As Object = Me.GetLabelContent(pos, actualSeries, actualXValues)
                                Dim dateTimeAxisLabel = New DateTimeAxisLabel(pos, obj, pos)

                                If (Math.Ceiling(VisibleRange.Start) < position) Then
                                    VisibleLabels.Add(dateTimeAxisLabel)
                                End If

                                date1 = xValue.AddDays(CType(interval, Integer)).Date

                            End If

                        End If

                        position = (position + 1)
                    Loop

                End If
            End If

        Else
            MyBase.GenerateVisibleLabels()
        End If

    End Sub

    Private Function GetLabelContent(ByVal pos As Integer, ByVal actualSeries As ChartSeriesBase, ByVal actualXValues As Object) As Object
        If (Not (actualSeries) Is Nothing) Then
            Dim xValues = CType(actualXValues, List(Of Double))
            If ((Not (xValues) Is Nothing) AndAlso (pos < xValues.Count) AndAlso (pos >= 0)) Then
                Dim xDateTime As DateTime = xValues(pos).FromOADate()
                Return xDateTime.ToString(Me.LabelFormat, Globalization.CultureInfo.CurrentCulture)
            Else
                Dim xStrValues = CType(actualXValues, List(Of String))
                If ((Not (xStrValues) Is Nothing) AndAlso pos < xStrValues.Count AndAlso pos >= 0) Then
                    Return xStrValues(pos)
                End If
            End If
        End If
        Return pos
    End Function

    'We need to update the secondary axis visible range after the primary axis actual range is updated.
    Protected Overrides Sub CalculateVisibleRange(avalableSize As Size)
        MyBase.CalculateVisibleRange(avalableSize)
        Dim area = CType(Me.GetType().GetProperty("Area", BindingFlags.GetProperty Or BindingFlags.NonPublic Or BindingFlags.Instance).GetValue(Me), SfChart)

        For Each axis In area.Axes

            Dim customSecondaryAxis As Object = Nothing

            If TypeOf axis Is CustomLogarithmicYAxis Or TypeOf axis Is CustomNumericalYAxis Then
                customSecondaryAxis = axis
            End If

            If Not customSecondaryAxis Is Nothing Then
                If customSecondaryAxis.IsUpdateNeeded Then

                    customSecondaryAxis.ZoomY(Me.VisibleRange, area)
                    customSecondaryAxis.IsUpdateNeeded = False
                End If

            End If
        Next

    End Sub

End Class

Class CustomLogarithmicYAxis
    Inherits LogarithmicAxis

    Public prevXVisibleRange As DoubleRange
    Public prevVisibleRange As DoubleRange
    Public IsUpdateNeeded As Boolean = True

    Protected Overrides Sub CalculateVisibleRange(avalableSize As Size)
        MyBase.CalculateVisibleRange(avalableSize)

        Me.VisibleRange = prevVisibleRange

        Dim area = CType(Me.GetType().GetProperty("Area", BindingFlags.GetProperty Or BindingFlags.NonPublic Or BindingFlags.Instance).GetValue(Me), SfChart)
        Dim primaryAxis = CType(area.PrimaryAxis, ChartAxisBase2D)
        Dim actualXRange = CType(primaryAxis.GetType().GetProperty("ActualRange", BindingFlags.GetProperty Or BindingFlags.NonPublic Or BindingFlags.Instance).GetValue(primaryAxis), DoubleRange)
        If actualXRange.IsEmpty Then Return
        Dim primaryAxisVisibleRange = CalculatePrimaryAxisRange(area, primaryAxis, actualXRange)

        If primaryAxisVisibleRange = prevXVisibleRange Then Return
        IsUpdateNeeded = True

        prevXVisibleRange = primaryAxisVisibleRange

    End Sub

    Private Function CalculatePrimaryAxisRange(area As SfChart, primaryAxis As ChartAxisBase2D, ByRef actualXRange As DoubleRange) As DoubleRange

        Dim rangeStart As Double
        rangeStart = actualXRange.Start + actualXRange.Delta * primaryAxis.ZoomPosition

        Dim rangeEnd As Double
        rangeEnd = rangeStart + primaryAxis.ZoomFactor * actualXRange.Delta

        Dim visibleXRange = New DoubleRange(rangeStart, rangeEnd)

        Return visibleXRange

    End Function

    Public Sub ZoomY(visibleXRange As DoubleRange, area As SfChart)
        Dim MinLogY, MaxLogY As Double
        Dim Ymax, Ymin As Double

        GetExtremes(visibleXRange.Start, visibleXRange.End, Ymax, Ymin, area, Me)

        MinLogY = Math.Log(Ymin, LogBase)
        MaxLogY = Math.Log(Ymax, LogBase)

        Me.VisibleRange = New DoubleRange(MinLogY, MaxLogY)

        prevVisibleRange = Me.VisibleRange

        '    Console.WriteLine("CustomLogarithmicYAxis " & Me.Name & "  ZoomY VisibleRange.Start " & Me.VisibleRange.Start & " VisibleRange.End " & Me.VisibleRange.End)
    End Sub

End Class

Class CustomNumericalYAxis
    Inherits NumericalAxis

    Public prevVisibleRange As DoubleRange
    Public IsUpdateNeeded As Boolean = True

    Protected Overrides Sub CalculateVisibleRange(avalableSize As Size)
        MyBase.CalculateVisibleRange(avalableSize)

        ' Console.WriteLine("CustomNumericalYAxis " & Me.Name & " CalculateVisibleRange VisibleRange.Start " & Me.VisibleRange.Start & " VisibleRange.End " & Me.VisibleRange.End)
        '  Console.WriteLine("CustomNumericalYAxis " & Me.Name & " CalculateVisibleRange prevVisibleRange.Start " & Me.prevVisibleRange.Start & " prevVisibleRange.End " & Me.prevVisibleRange.End)

        Me.VisibleRange = prevVisibleRange

    End Sub


    Public Sub ZoomY(visibleXRange As DoubleRange, area As SfChart)
        Dim Ymax, Ymin As Double

        GetExtremes(visibleXRange.Start, visibleXRange.End, Ymax, Ymin, area, Me)

        Me.VisibleRange = New DoubleRange(0, Ymax)

        prevVisibleRange = Me.VisibleRange

        ' Console.WriteLine("CustomNumericalYAxis " & Me.Name & "  ZoomY VisibleRange.Start " & Me.VisibleRange.Start & " VisibleRange.End " & Me.VisibleRange.End)
    End Sub

End Class

Class FastSteplineBitmapSeriesExt
    Inherits FastStepLineBitmapSeries
    Protected Overrides Sub SetIndividualPoint(index As Integer, obj As Object, replace As Boolean)
        MyBase.SetIndividualPoint(index, obj, replace)
        CType(Me.YAxis, Object).IsUpdateNeeded = True
    End Sub
End Class

Class AreaSeriesExt
    Inherits AreaSeries
    Protected Overrides Sub SetIndividualPoint(index As Integer, obj As Object, replace As Boolean)
        MyBase.SetIndividualPoint(index, obj, replace)
        CType(Me.YAxis, Object).IsUpdateNeeded = True
    End Sub
End Class

Class FastRangeAreaBitmapSeriesExt
    Inherits FastRangeAreaBitmapSeries
    Protected Overrides Sub SetIndividualPoint(index As Integer, obj As Object, replace As Boolean)
        MyBase.SetIndividualPoint(index, obj, replace)
        CType(Me.YAxis, Object).IsUpdateNeeded = True
    End Sub
End Class